@@ -1,15 +1,7 @@
-// Copyright 2018 Evgeniy Reizner
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 use std::iter::FilterMap;
 use std::cell::{Ref, RefMut};
 
-use {
+use crate::{
     tree,
     Attribute,
     AttributeId,
@@ -276,7 +268,7 @@ impl Node {
     /// # Panics
     ///
     /// Panics if the node is currently mutably borrowed.
-    pub fn is_tag_name<'a, T>(&self, tag_name: T) -> bool
+    pub fn has_tag_name<'a, T>(&self, tag_name: T) -> bool
         where TagNameRef<'a>: From<T>
     {
         self.borrow().tag_name.as_ref() == TagNameRef::from(tag_name)
